/* ============================================================
 *
 * This file is part of the KDE project
 *
 * Date        : 2007-09-13
 * Description : Sane interface for KDE
 *
 * Copyright (C) 2009 by Grzegorz Kurtyka <grzegorz dot kurtyka at gmail dot com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) version 3, or any
 * later version accepted by the membership of KDE e.V. (or its
 * successor approved by the membership of KDE e.V.), which shall
 * act as a proxy defined in Section 6 of version 3 of the license.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ============================================================ */

#include "sane_find_devices_thread.h"
#include "sane_find_devices_thread.moc"

// Qt includes.
#include <QDebug>


namespace LibQSane
{

FindSaneDevicesThread::FindSaneDevicesThread(QObject *parent)
     : QThread(parent)
{
}

FindSaneDevicesThread::~FindSaneDevicesThread()
{
    wait();
}


void FindSaneDevicesThread::run()
{
    m_devicesMap.clear();

    SANE_Device const **dev_list;
    SANE_Status status = sane_get_devices(&dev_list, SANE_FALSE);
    if (status != SANE_STATUS_GOOD)
        return;

    int i = 0;
    while(dev_list[i] != 0)
    {
        SaneDeviceInfo deviceInfo;
        deviceInfo.name   = QString::fromLocal8Bit(dev_list[i]->name);
        deviceInfo.vendor = QString::fromLocal8Bit(dev_list[i]->vendor);
        deviceInfo.model  = QString::fromLocal8Bit(dev_list[i]->model);
        deviceInfo.type   = QString::fromLocal8Bit(dev_list[i]->type);
        m_devicesMap.insert( dev_list[i]->name, deviceInfo );
        i++;
    }
}

void FindSaneDevicesThread::getDevicesList(QMap<QString, SaneDeviceInfo> &devicesList) {
    devicesList = m_devicesMap;
}

}
