/*
    <one line to give the library's name and an idea of what it does.>
    Copyright (C) <year>  <name of author>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*/

#include "sane_options_widget.h"
#include "sane_options_widget.moc"
#include "sane_options_widget_private.h"

#include <QDebug>

#include <QTabWidget>
#include <QScrollArea>
#include <QVBoxLayout>
#include <QCheckBox>
#include <QScrollBar>
#include <QTimer>
#include <QMap>

#include "labeled_gamma.h"
#include "sane_option.h"

namespace LibQSane
{

SaneOptionsWidget::SaneOptionsWidget(QWidget *parent)
    : QWidget(parent),
      d_ptr(new SaneOptionsWidgetPrivate())
{
    Q_D(SaneOptionsWidget);
    d->q_ptr = this;

    d->init();
}

SaneOptionsWidget::SaneOptionsWidget(SaneOptionsWidgetPrivate& dd, QWidget* parent)
    : QWidget(parent),
      d_ptr(&dd)
{
    Q_D(SaneOptionsWidget);
    d->q_ptr = this;

    d->init();
}

void SaneOptionsWidget::clearDeviceOptions()
{
    Q_D(SaneOptionsWidget);
    
    d->clearDeviceOptions();
}

void SaneOptionsWidget::setDefaultValues()
{
    Q_D(SaneOptionsWidget);
    
    SaneOption *option;
    
    // Try to get Color mode by default
    if ((option = d->getOption(SANE_NAME_SCAN_MODE)) != 0) {
        option->setRawValue(SANE_VALUE_SCAN_MODE_COLOR);
    }
    
    // Try to set 8 bit color
    if ((option = d->getOption(SANE_NAME_BIT_DEPTH)) != 0) {
        option->setValue(8);
    }
    
    // Try to set Scan resolution to 600 DPI
    if (d->m_optRes != 0) {
        d->m_optRes->setValue(600);
    }
}

int SaneOptionsWidget::setOptVals(const QMap <QString, QByteArray> &opts)
{
    Q_D(SaneOptionsWidget);
    
    QByteArray tmp;
    int i;
    int ret=0;

    for (i=0; i<d->m_optList.size(); i++)
    {
        SaneOption* option = d->m_optList.at(i);
        if (d->isScanAreaOption(option))
            continue;
        
        if (opts.contains(option->name())) {
            tmp = opts[option->name()];
            if (option->setRawValue(tmp)) {
                ret++;
            }
        }
    }
    if ((d->m_splitGamChB) &&
        (d->m_optGamR) &&
        (d->m_optGamG) &&
        (d->m_optGamB))
    {
        // check if the current gamma values are identical. if they are identical,
        // uncheck the "Separate color intensity tables" checkbox
        QString redGamma;
        QString greenGamma;
        QString blueGamma;
        d->m_optGamR->getValue(redGamma);
        d->m_optGamG->getValue(greenGamma);
        d->m_optGamB->getValue(blueGamma);
        if ((redGamma == greenGamma) && (greenGamma == blueGamma)) {
            d->m_splitGamChB->setChecked(false);
            // set the values to the common gamma widget
            d->m_commonGamma->setValues(redGamma);
        }
        else {
            d->m_splitGamChB->setChecked(true);
        }
    }
    
    return ret;
}

QMap<QString, QByteArray> SaneOptionsWidget::getOptVals()
{
    Q_D(SaneOptionsWidget);
    
    QMap<QString, QByteArray> opts;
    
    SaneOption *option;
    QByteArray tmp;

    for (int i=1; i<d->m_optList.size(); i++) {
        option = d->m_optList.at(i);
        if (d->isScanAreaOption(option))
            continue;
        
        if (option->getRawValue(tmp)) {
            opts[option->name()] = tmp;
        }
    }
    
    return opts;
}

bool SaneOptionsWidget::setOptVal(const QString &option, const QByteArray &value)
{
    Q_D(SaneOptionsWidget);
    
    SaneOption *opt;

    if ((opt = d->getOption(option)) != 0) {
        if (opt->setRawValue(value)) {
            if ((d->m_splitGamChB) &&
                (d->m_optGamR) &&
                (d->m_optGamG) &&
                (d->m_optGamB) &&
                ((opt == d->m_optGamR) ||
                (opt == d->m_optGamG) ||
                (opt == d->m_optGamB)))
            {
                // check if the current gamma values are identical. if they are identical,
                // uncheck the "Separate color intensity tables" checkbox
                QString redGamma;
                QString greenGamma;
                QString blueGamma;
                d->m_optGamR->getValue(redGamma);
                d->m_optGamG->getValue(greenGamma);
                d->m_optGamB->getValue(blueGamma);
                if ((redGamma == greenGamma) && (greenGamma == blueGamma)) {
                    d->m_splitGamChB->setChecked(false);
                    // set the values to the common gamma widget
                    d->m_commonGamma->setValues(redGamma);
                }
                else {
                    d->m_splitGamChB->setChecked(true);
                }
            }
            return true;
        }
    }

    return false;
}

bool SaneOptionsWidget::getOptVal(const QString &optname, QByteArray &value)
{
    Q_D(SaneOptionsWidget);
    
    SaneOption *option;

    if ((option = d->getOption(optname)) != 0) {
        return option->getRawValue(value);
    }
    return false;
}

float SaneOptionsWidget::currentDPI()
{
    Q_D(SaneOptionsWidget);
    
    if (d->m_optRes) {
        float value;
        if (d->m_optRes->getValue(value)) {
            return value;
        }
    }
    return 0.0;
}

bool SaneOptionsWidget::invertColors() const
{
    Q_D(const SaneOptionsWidget);
    return d->m_invertColors->isChecked();
}

bool SaneOptionsWidget::isADFSource() const
{
    Q_D(const SaneOptionsWidget);
    if (d->m_optSource)
    {
            QByteArray source;
            d->m_optSource->getRawValue(source);

            if (source == "ADF" || source == "Automatic Document Feeder")
                return true;
    }
    
    return false;
}

bool SaneOptionsWidget::waitForButton() const
{
    Q_D(const SaneOptionsWidget);
    if (d->m_optWaitForBtn)
    {
            QByteArray wait;
            d->m_optWaitForBtn->getRawValue(wait);

            if (wait == "true")
                return true;
    }
    
    return false;
}

bool SaneOptionsWidget::setScanResolution(float dpi)
{
    Q_D(const SaneOptionsWidget);
    if (d->m_optRes == 0)
        return false;
    
    if (dpi >= 25.0) {
        d->m_optRes->setValue(dpi);
        if ((d->m_optResY != 0) && (d->m_optRes->name() == SANE_NAME_SCAN_X_RESOLUTION)) {
            d->m_optResY->setValue(dpi);
        }
    }
    else {
        // set the resolution to getMinValue and increase if necessary
        SANE_Parameters params;
        float dpi;
        d->m_optRes->getMinValue(dpi);
        do {
            d->m_optRes->setValue(dpi);
            if ((d->m_optResY != 0) && (d->m_optRes->name() == SANE_NAME_SCAN_X_RESOLUTION)) {
                d->m_optResY->setValue(dpi);
            }
            //check what image size we would get in a scan
            SANE_Status status = sane_get_parameters(d->m_saneHandle, &params);
            if (status != SANE_STATUS_GOOD) {
                qDebug() << "sane_get_parameters=" << sane_strstatus(status);
                return false;
            }
            
            if (dpi > 600) break;
        
            // Increase the dpi value
            dpi += 25.0;
        }
        while ((params.pixels_per_line < 300) || ((params.lines > 0) && (params.lines < 300)));
        
        if (params.pixels_per_line == 0) {
            // This is a security measure for broken backends
            d->m_optRes->getMinValue(dpi);
            d->m_optRes->setValue(dpi);
            qDebug() << "Setting minimum DPI value for a broken back-end"; 
        }
    }
    
    return true;
}

bool SaneOptionsWidget::getScanResolution(float* dpi) const
{
    Q_D(const SaneOptionsWidget);
    if (!d->m_optRes)
        return false;
    
    return d->m_optRes->getValue(*dpi);
}

bool SaneOptionsWidget::setPreviewMode(bool enable)
{
    Q_D(SaneOptionsWidget);
    if (d->m_optPreview != 0)
    {
        return d->m_optPreview->setValue(enable ? SANE_TRUE : SANE_FALSE);
    }
    
    return false;
}

void SaneOptionsWidget::createOptInterface(QList<SaneOption*> options, SANE_Handle saneHandle)
{
    Q_D(SaneOptionsWidget);
    
    d->m_saneHandle = saneHandle;
    d->m_optList = options;
    
    // do the connections of the option parameters
    for (int i=1; i<d->m_optList.size(); ++i) {
        //qDebug() << d->m_optList.at(i)->name();
        connect (d->m_optList.at(i), SIGNAL(optsNeedReload()), d, SLOT(optReload()));
        connect (d->m_optList.at(i), SIGNAL(valsNeedReload()), d, SLOT(scheduleValReload()));
    }
    
    d->createOptInterface();
}

void SaneOptionsWidget::flushValuesReloading()
{
    Q_D(SaneOptionsWidget);
    
    while (d->m_readValsTmr.isActive()) {
        d->m_readValsTmr.stop();
        d->valReload();
    }
}

void SaneOptionsWidget::storePreviewOptions()
{
    Q_D(SaneOptionsWidget);
    
    if (d->m_optDepth != 0) d->m_optDepth->storeCurrentData();
    if (d->m_optRes != 0) d->m_optRes->storeCurrentData();
    if (d->m_optResY != 0) d->m_optResY->storeCurrentData();
    if (d->m_optPreview != 0) d->m_optPreview->storeCurrentData();
}

void SaneOptionsWidget::restorePreviewOptions()
{
    Q_D(SaneOptionsWidget);
    
    if (d->m_optDepth != 0) d->m_optDepth->restoreSavedData();
    if (d->m_optRes != 0) d->m_optRes->restoreSavedData();
    if (d->m_optResY != 0) d->m_optResY->restoreSavedData();
    if (d->m_optPreview != 0) d->m_optPreview->restoreSavedData();
}

float SaneOptionsWidget::scanAreaWidth()
{
    Q_D(SaneOptionsWidget);
    
    float result = 0.0;
    
    if (d->m_optBrX) {
        if (d->m_optBrX->getUnit() == SANE_UNIT_PIXEL) {
            d->m_optBrX->getMaxValue(result);
            result = result / currentDPI() / 25.4;
        }
        else if (d->m_optBrX->getUnit() == SANE_UNIT_MM) {
            d->m_optBrX->getMaxValue(result);
        }
    }
    return result;
}

float SaneOptionsWidget::scanAreaHeight()
{
    Q_D(SaneOptionsWidget);
    
    float result = 0.0;
    
    if (d->m_optBrY) {
        if (d->m_optBrY->getUnit() == SANE_UNIT_PIXEL) {
            d->m_optBrY->getMaxValue(result);
            result = result / currentDPI() / 25.4;
        }
        else if (d->m_optBrY->getUnit() == SANE_UNIT_MM) {
            d->m_optBrY->getMaxValue(result);
        }
    }
    return result;
}

bool SaneOptionsWidget::canSetScanArea() const
{
    Q_D(const SaneOptionsWidget);
    return d->m_optBrX && d->m_optBrY && d->m_optTlX && d->m_optTlY;
}

bool SaneOptionsWidget::setScanArea(QPointF topLeft, QPointF bottomRight)
{
    Q_D(SaneOptionsWidget);
    if (!canSetScanArea())
        return false;
    
    if (topLeft.x() < 0.0 || topLeft.y() < 0.0 || bottomRight.x() < 0.0 || bottomRight.y() < 0.0)
        return false;
    
    float xmax, ymax;
    d->m_optBrX->getMaxValue(xmax);
    d->m_optBrY->getMaxValue(ymax);
    
    if (d->m_optBrY->getUnit() == SANE_UNIT_MM)
    {
        // clear selection if values are out of bounds
        if (topLeft.x() > xmax || topLeft.y() > ymax || bottomRight.x() > xmax || bottomRight.y() > ymax)
            return false;
        
        d->m_optTlX->setValue(topLeft.x());
        d->m_optTlY->setValue(topLeft.y());
        d->m_optBrX->setValue(bottomRight.x());
        d->m_optBrY->setValue(bottomRight.y());
    }
    else if (d->m_optBrY->getUnit() == SANE_UNIT_PIXEL)
    {
        const float mmperinch = 25.4;
        const float dpi = currentDPI();
        const float m = dpi / mmperinch;
        if (m*topLeft.x() > xmax || m*topLeft.y() > ymax || m*bottomRight.x() > xmax || m*bottomRight.y() > ymax)
            return false;
        
        d->m_optTlX->setValue(m*topLeft.x());
        d->m_optTlY->setValue(m*topLeft.y());
        d->m_optBrX->setValue(m*bottomRight.x());
        d->m_optBrY->setValue(m*bottomRight.y());
    }
    
    return true;
}

bool SaneOptionsWidget::setScanAreaByRatios(float tl_x, float tl_y, float br_x, float br_y)
{
    Q_D(SaneOptionsWidget);
    if (!canSetScanArea())
        return false;
    
    if (tl_x < 0.0 || tl_y < 0.0 || br_x < 0.0 || br_y < 0.0)
        return false;
    
    if (tl_x > 1.0 || tl_y > 1.0 || br_x > 1.0 || br_y > 1.0)
        return false;
    
    float max_x;
    float max_y;
    
    d->m_optBrX->getMaxValue(max_x);
    d->m_optBrY->getMaxValue(max_y);
    
    //qDebug() << Q_FUNC_INFO << "max_x =" << max_x << ", max_y =" << max_y << ", max_tlx =" << max_tlx << ", max_tly = " << max_tly;
    
    d->m_optTlX->setValue(tl_x * max_x);
    d->m_optTlY->setValue(tl_y * max_y);
    d->m_optBrX->setValue(br_x * max_x);
    d->m_optBrY->setValue(br_y * max_y);
    
    return true;
}

void SaneOptionsWidget::setFullScanArea()
{
    Q_D(SaneOptionsWidget);
    if (!canSetScanArea())
        return;
    
    float xmax, ymax;
    d->m_optBrX->getMaxValue(xmax);
    d->m_optBrY->getMaxValue(ymax);
    
    d->m_optTlX->setValue(0.0);
    d->m_optTlY->setValue(0.0);
    d->m_optBrX->setValue(xmax);
    d->m_optBrY->setValue(ymax);
}

void SaneOptionsWidget::reloadOptions()
{
    Q_D(SaneOptionsWidget);
    d->optReload();
}

}
