/*
    <one line to give the library's name and an idea of what it does.>
    Copyright (C) <year>  <name of author>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef SANE_OPTIONS_WIDGET_H
#define SANE_OPTIONS_WIDGET_H

#include "libqsane_export.h"

extern "C"
{
    #include <sane/sane.h>
}

#include <QWidget>

namespace LibQSane
{

class SaneOption;

class SaneOptionsWidgetPrivate;

class LIBQSANE_EXPORT SaneOptionsWidget : public QWidget
{
    Q_OBJECT
public:
    SaneOptionsWidget(QWidget* parent = 0);
    
    void clearDeviceOptions();
    void createOptInterface(QList<SaneOption*> options, SANE_Handle saneHandle);
    
    void setDefaultValues();
    
    int setOptVals(const QMap <QString, QByteArray> &opts);
    QMap<QString, QByteArray> getOptVals();
    bool setOptVal(const QString &optname, const QByteArray &value);
    bool getOptVal(const QString &optname, QByteArray &value);
    
    float currentDPI();
    bool invertColors() const;
    bool isADFSource() const;
    bool waitForButton() const;
    
    bool setScanResolution(float dpi);
    bool getScanResolution(float* dpi) const;
    
    bool setPreviewMode(bool enable = true);
    
    /** This method returns the scan area's width in mm
    * @return Width of the scannable area in mm */
    float scanAreaWidth();

    /** This method returns the scan area's height in mm
    * @return Height of the scannable area in mm */
    float scanAreaHeight();
    
    /** This method determines if scanner allows to set scan area */
    bool canSetScanArea() const;
    
    /** This method sets the scan area
    * topLeft and bottomRight parameters are in mm
    * @return true if can set the scan area and all parameters are right */
    bool setScanArea(QPointF topLeft, QPointF bottomRight);
    
    /** This method sets the scan area
    * tl_x, tl_y, br_x, br_y must be in the range 0.0 -> 1.0
    * @return true if can set the scan area and all parameters are right */
    bool setScanAreaByRatios(float tl_x, float tl_y, float br_x, float br_y);
    
    /** This method maximizes the scan area */
    void setFullScanArea();
    
    void flushValuesReloading();
    
    void storePreviewOptions();
    void restorePreviewOptions();
    
    void reloadOptions();
    
signals:
    void optionsReloaded();
    void invertColorsToggled(bool);
    
    void tlXChanged(float ratio);
    void tlYChanged(float ratio);
    void brXChanged(float ratio);
    void brYChanged(float ratio);
    
    void scanSourceChanged();
    
protected:
    QScopedPointer<SaneOptionsWidgetPrivate> d_ptr;
    SaneOptionsWidget(SaneOptionsWidgetPrivate& dd, QWidget* parent = 0);

private:
    Q_DECLARE_PRIVATE(SaneOptionsWidget);
};

}

#endif // SANE_OPTIONS_WIDGET_H
