/*
    <one line to give the library's name and an idea of what it does.>
    Copyright (C) <year>  <name of author>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*/

#include "sane_options_widget_private.h"
#include "sane_options_widget_private.moc"

#include "sane_options_widget.h"

#include <QDebug>

#include <QVBoxLayout>
#include <QTabWidget>
#include <QScrollArea>
#include <QCheckBox>
#include <QScrollBar>
#include <QLabel>

#include "sane_option.h"
#include "labeled_gamma.h"
#include "sane_i18n.h"

namespace LibQSane
{

SaneOptionsWidgetPrivate::SaneOptionsWidgetPrivate()
{
    m_optsTabWidget = 0;
    m_basicScrollA  = 0;
    m_basicOptsTab  = 0;
    m_otherScrollA  = 0;
    m_otherOptsTab  = 0;
    m_colorOpts     = 0;
    m_invertColors  = 0;
    
    m_splitGamChB   = 0;
    m_commonGamma   = 0;
    
    m_saneHandle = 0;
    
    clearDeviceOptions();
    
    m_readValsTmr.setSingleShot(true);
    connect(&m_readValsTmr,   SIGNAL(timeout()), this, SLOT(valReload()));
}

SaneOptionsWidgetPrivate::~SaneOptionsWidgetPrivate()
{
}

void SaneOptionsWidgetPrivate::init()
{
    Q_Q(SaneOptionsWidget);
    
    QVBoxLayout* layout = new QVBoxLayout;
    layout->setContentsMargins(0, 0, 0, 0);
    
    m_optsTabWidget = new QTabWidget;
    m_optsTabWidget->setTabPosition(QTabWidget::West);
    
    // Add the basic options tab
    m_basicScrollA = new QScrollArea;
    m_basicScrollA->setWidgetResizable(true);
    m_basicScrollA->setFrameShape(QFrame::NoFrame);
    m_optsTabWidget->addTab(m_basicScrollA, SaneOptionsWidget::tr("Basic Options"));

    // Add the other options tab
    m_otherScrollA = new QScrollArea;
    m_otherScrollA->setWidgetResizable(true);
    m_otherScrollA->setFrameShape(QFrame::NoFrame);
    m_optsTabWidget->addTab(m_otherScrollA, SaneOptionsWidget::tr("Advanced Options"));
    
    layout->addWidget(m_optsTabWidget);
    q->setLayout(layout);
}

SaneOption* SaneOptionsWidgetPrivate::getOption(const QString &name)
{
    int i;
    for (i=0; i<m_optList.size(); i++) {
        if (m_optList.at(i)->name() == name) {
            return m_optList.at(i);
        }
    }
    return 0;
}

void SaneOptionsWidgetPrivate::clearDeviceOptions()
{
    m_optSource     = 0;
    m_optNegative   = 0;
    m_optFilmType   = 0;
    m_optMode       = 0;
    m_optDepth      = 0;
    m_optRes        = 0;
    m_optResY       = 0;
    m_optTlX        = 0;
    m_optTlY        = 0;
    m_optBrX        = 0;
    m_optBrY        = 0;
    m_optGamR       = 0;
    m_optGamG       = 0;
    m_optGamB       = 0;
    m_optPreview    = 0;
    m_optWaitForBtn = 0;
    
    delete m_invertColors;
    m_invertColors  = 0;
    
    delete m_basicOptsTab;
    m_basicOptsTab = 0;

    delete m_otherOptsTab;
    m_otherOptsTab = 0;
    
    delete m_commonGamma;
    m_commonGamma = 0;

    delete m_splitGamChB;
    m_splitGamChB = 0;
    
    // delete all the options in the list.
    while (!m_optList.isEmpty()) {
        delete m_optList.takeFirst();
    }
}

void SaneOptionsWidgetPrivate::createOptInterface()
{
    Q_Q(SaneOptionsWidget);
    
    m_basicOptsTab = new QWidget;
    m_basicScrollA->setWidget(m_basicOptsTab);

    QGridLayout *basic_layout = new QGridLayout(m_basicOptsTab);
    SaneOption *option;
    // Scan Source
    if ((option = getOption(SANE_NAME_SCAN_SOURCE)) != 0) {
        m_optSource = option;
        addRowToGridLayout(basic_layout, option);
        connect(m_optSource, SIGNAL(valueChanged()), this, SLOT(checkInvert()), Qt::QueuedConnection);
        connect(m_optSource, SIGNAL(valueChanged()), q, SIGNAL(scanSourceChanged()));
    }
    // film-type (note: No translation)
    if ((option = getOption(QString("film-type"))) != 0) {
        m_optFilmType = option;
        addRowToGridLayout(basic_layout, option);
        connect(m_optFilmType, SIGNAL(valueChanged()), this, SLOT(checkInvert()), Qt::QueuedConnection);
    }
    else if ((option = getOption(SANE_NAME_NEGATIVE)) != 0) {
        m_optNegative = option;
        addRowToGridLayout(basic_layout, option);
    }
    // Scan mode
    if ((option = getOption(SANE_NAME_SCAN_MODE)) != 0) {
        m_optMode = option;
        addRowToGridLayout(basic_layout, option);
    }
    // Bitdepth
    if ((option = getOption(SANE_NAME_BIT_DEPTH)) != 0) {
        m_optDepth = option;
        addRowToGridLayout(basic_layout, option);
    }
    // Threshold
    if ((option = getOption(SANE_NAME_THRESHOLD)) != 0) {
        addRowToGridLayout(basic_layout, option);
    }
    // Resolution
    if ((option = getOption(SANE_NAME_SCAN_RESOLUTION)) != 0) {
        m_optRes = option;
        addRowToGridLayout(basic_layout, option);
    }
    else {
        // TODO: add a combined x and y resolution widget.
        if ((option = getOption(SANE_NAME_SCAN_X_RESOLUTION)) != 0) {
        m_optRes = option;
        addRowToGridLayout(basic_layout, option);
        }
        if ((option = getOption(SANE_NAME_SCAN_Y_RESOLUTION)) != 0) {
            m_optResY = option;
            addRowToGridLayout(basic_layout, option);
        }
    }

    // save a pointer to the preview option if possible
    if ((option = getOption(SANE_NAME_PREVIEW)) != 0) {
        m_optPreview = option;
    }

    // save a pointer to the "wait-for-button" option if possible (Note: No translation)
    if ((option = getOption("wait-for-button")) != 0) {
        m_optWaitForBtn = option;
    }

    // scan area (Do not add the widgets)
    if ((option = getOption(SANE_NAME_SCAN_TL_X)) != 0) {
        m_optTlX = option;
        connect(m_optTlX, SIGNAL(fValueRead(float)), this, SLOT(slotTlXChanged(float)));
    }
    if ((option = getOption(SANE_NAME_SCAN_TL_Y)) != 0) {
        m_optTlY = option;
        connect(m_optTlY, SIGNAL(fValueRead(float)), this, SLOT(slotTlYChanged(float)));
    }
    if ((option = getOption(SANE_NAME_SCAN_BR_X)) != 0) {
        m_optBrX = option;
        connect(m_optBrX, SIGNAL(fValueRead(float)), this, SLOT(slotBrXChanged(float)));
    }
    if ((option = getOption(SANE_NAME_SCAN_BR_Y)) != 0) {
        m_optBrY = option;
        connect(m_optBrY, SIGNAL(fValueRead(float)), this, SLOT(slotBrYChanged(float)));
    }

    // Color Options Frame
    m_colorOpts = new QWidget;
    addRowToGridLayout(basic_layout, m_colorOpts);
    QGridLayout *color_lay = new QGridLayout(m_colorOpts);
    color_lay->setContentsMargins(0, 9, 0, 9);

    // Add Color correction to the color "frame"
    if ((option = getOption(SANE_NAME_BRIGHTNESS)) != 0) {
        addRowToGridLayout(color_lay, option);
    }
    if ((option = getOption(SANE_NAME_CONTRAST)) != 0) {
        addRowToGridLayout(color_lay, option);
    }
    
    // Add gamma tables to the color "frame"
    QWidget *gamma_frm = new QWidget;
    addRowToGridLayout(color_lay, gamma_frm);
    QGridLayout *gam_frm_l = new QGridLayout(gamma_frm);
    gam_frm_l->setContentsMargins(0, 9, 0, 9);

    if ((option = getOption(SANE_NAME_GAMMA_VECTOR_R)) != 0) {
        m_optGamR= option;
        addRowToGridLayout(gam_frm_l, option);
    }
    if ((option = getOption(SANE_NAME_GAMMA_VECTOR_G)) != 0) {
        m_optGamG= option;
        addRowToGridLayout(gam_frm_l, option);
    }
    if ((option = getOption(SANE_NAME_GAMMA_VECTOR_B)) != 0) {
        m_optGamB= option;
        addRowToGridLayout(gam_frm_l, option);
    }
    
    
    if ((m_optGamR != 0) && (m_optGamG != 0) && (m_optGamB != 0)) {
        LabeledGamma *gamma = reinterpret_cast<LabeledGamma *>(m_optGamR->widget());
        m_commonGamma = new LabeledGamma(m_colorOpts, sane_i18n(SANE_TITLE_GAMMA_VECTOR), gamma->size());
        m_commonGamma->setToolTip(sane_i18n(SANE_DESC_GAMMA_VECTOR));
        
        addRowToGridLayout(color_lay, m_commonGamma);
        
        connect(m_commonGamma, SIGNAL(gammaChanged(int,int,int)), m_optGamR->widget(), SLOT(setValues(int,int,int)));
        connect(m_commonGamma, SIGNAL(gammaChanged(int,int,int)), m_optGamG->widget(), SLOT(setValues(int,int,int)));
        connect(m_commonGamma, SIGNAL(gammaChanged(int,int,int)), m_optGamB->widget(), SLOT(setValues(int,int,int)));

        m_splitGamChB = new QCheckBox(tr("Separate color intensity tables"), m_basicOptsTab);
        addRowToGridLayout(color_lay, m_splitGamChB);
        
        connect (m_splitGamChB, SIGNAL(toggled(bool)), gamma_frm, SLOT(setVisible(bool)));
        connect (m_splitGamChB, SIGNAL(toggled(bool)), m_commonGamma, SLOT(setHidden(bool)));
        
        gamma_frm->hide();
    }
    
    if ((option = getOption(SANE_NAME_BLACK_LEVEL)) != 0) {
        addRowToGridLayout(color_lay, option);
    }
    if ((option = getOption(SANE_NAME_WHITE_LEVEL)) != 0) {
        addRowToGridLayout(color_lay, option);
    }
    
    m_invertColors = new QCheckBox(tr("Invert colors"), m_colorOpts);
    addRowToGridLayout(color_lay, m_invertColors);
    m_invertColors->setChecked(false);
    connect(m_invertColors, SIGNAL(toggled(bool)), q, SIGNAL(invertColorsToggled(bool)));
    
    // add a stretch to the end to keep the parameters at the top
    basic_layout->addItem(new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding), basic_layout->rowCount(), 0);

    
    // Remaining (un known) options go to the "Other Options"
    m_otherOptsTab = new QWidget;
    m_otherScrollA->setWidget(m_otherOptsTab);
    
    QGridLayout *other_layout = new QGridLayout(m_otherOptsTab);
    
    // add the remaining parameters
    for (int i=0; i<m_optList.size(); i++) {
        if ((m_optList.at(i)->widget() == 0) &&
            (m_optList.at(i)->name() != SANE_NAME_SCAN_TL_X) &&
            (m_optList.at(i)->name() != SANE_NAME_SCAN_TL_Y) &&
            (m_optList.at(i)->name() != SANE_NAME_SCAN_BR_X) &&
            (m_optList.at(i)->name() != SANE_NAME_SCAN_BR_Y) &&
            (m_optList.at(i)->name() != SANE_NAME_PREVIEW) &&
            (m_optList.at(i)->hasGui()))
        {
            addRowToGridLayout(other_layout, m_optList.at(i));
        }
    }
    
    // add a stretch to the end to keep the parameters at the top
    other_layout->addItem(new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding), other_layout->rowCount(), 0);
    
    // encsure that we do not get a scrollbar at the bottom of the option of the options
    int min_width = m_basicOptsTab->sizeHint().width();
    if (min_width < m_otherOptsTab->sizeHint().width()) {
        min_width = m_otherOptsTab->sizeHint().width();
    }
    
    m_optsTabWidget->setMinimumWidth(min_width +
    m_basicScrollA->verticalScrollBar()->sizeHint().width() + 5);
}

void SaneOptionsWidgetPrivate::checkInvert()
{   
    if (!m_optSource) return;
    if (!m_optFilmType) return;
    //TODO: if (m_scanOngoing) return;
    
    QByteArray source;
    QByteArray filmtype;
    m_optSource->getRawValue(source);
    m_optFilmType->getRawValue(filmtype);
    
    //: This is compared to the option string returned by sane
    if ((source.toLower().contains("transparency")) &&
    //: This is compared to the option string returned by sane
        (filmtype.toLower().contains("negative")))
    {
        m_invertColors->setChecked(true);
    }
    else {
        m_invertColors->setChecked(false);
    }
}

void SaneOptionsWidgetPrivate::scheduleValReload()
{
    m_readValsTmr.start(5);
}

void SaneOptionsWidgetPrivate::optReload()
{
    qDebug() << Q_FUNC_INFO;
    Q_Q(SaneOptionsWidget);
    
    for (int i=0; i<m_optList.size(); i++) {
        m_optList.at(i)->readOption();
        // Also read the values
        m_optList.at(i)->readValue();
    }
    
    // ensure that we do not get a scrollbar at the bottom of the option of the options
    int min_width = m_basicOptsTab->sizeHint().width();
    if (min_width < m_otherOptsTab->sizeHint().width()) {
        min_width = m_otherOptsTab->sizeHint().width();
    }
    
    q->setMinimumWidth(min_width + 32);
    
    emit q->optionsReloaded();
}

void SaneOptionsWidgetPrivate::valReload()
{
    int i;
    QString tmp;
    
    for (i=0; i<m_optList.size(); i++) {
        m_optList.at(i)->readValue();
    }
    
}

void SaneOptionsWidgetPrivate::addRowToGridLayout(QGridLayout* l, QWidget* w, QWidget* labelWidget)
{
    int row = l->rowCount();
    
    if (!labelWidget)
        //l->addWidget(w, row, 0, 1, 2);
        l->addWidget(w, row, 0);
    else
    {
        l->addWidget(labelWidget, row, 0);
        //l->addWidget(w, row, 1);
        l->addWidget(w, row+1, 0);
    }
}

void SaneOptionsWidgetPrivate::addRowToGridLayout(QGridLayout* l, SaneOption* option)
{
    Q_Q(SaneOptionsWidget);
    
    option->createLabelWidget(q);
    option->createWidget(q);
    
    addRowToGridLayout(l, option->widget(), option->labelWidget());
}

void SaneOptionsWidgetPrivate::slotTlXChanged(float tlx)
{
    Q_Q(SaneOptionsWidget);
    float max;
    m_optTlX->getMaxValue(max);
    emit q->tlXChanged(tlx / max);
}

void SaneOptionsWidgetPrivate::slotTlYChanged(float tly)
{
    Q_Q(SaneOptionsWidget);
    float max;
    m_optTlY->getMaxValue(max);
    emit q->tlYChanged(tly / max);
}

void SaneOptionsWidgetPrivate::slotBrXChanged(float brx)
{
    Q_Q(SaneOptionsWidget);
    float max;
    m_optBrX->getMaxValue(max);
    emit q->brXChanged(brx / max);
}

void SaneOptionsWidgetPrivate::slotBrYChanged(float bry)
{
    Q_Q(SaneOptionsWidget);
    float max;
    m_optBrY->getMaxValue(max);
    emit q->brYChanged(bry / max);
}

bool SaneOptionsWidgetPrivate::isScanAreaOption(const SaneOption* option) const
{
    return (option == m_optTlX) || (option == m_optTlY) || (option == m_optBrX) || (option == m_optBrY);
}

}
