/*
    <one line to give the library's name and an idea of what it does.>
    Copyright (C) <year>  <name of author>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef SANE_OPTIONS_WIDGET_PRIVATE_H
#define SANE_OPTIONS_WIDGET_PRIVATE_H

#include <QObject>
#include <QTimer>

extern "C"
{
    #include <sane/sane.h>
}

class QTabWidget;
class QScrollArea;
class QCheckBox;
class QGridLayout;

namespace LibQSane
{

class SaneOptionsWidget;
class SaneOption;
class LabeledGamma;

class SaneOptionsWidgetPrivate : public QObject
{
    Q_OBJECT
    Q_DECLARE_PUBLIC(SaneOptionsWidget);
public:
    SaneOptionsWidget* q_ptr;
    
    SaneOptionsWidgetPrivate();
    virtual ~SaneOptionsWidgetPrivate();
    
    void init();
    SaneOption *getOption(const QString &name);
    
    void clearDeviceOptions();
    void createOptInterface();
    
    bool isScanAreaOption(const SaneOption* option) const;
    
    QTabWidget         *m_optsTabWidget;
    QScrollArea        *m_basicScrollA;
    QWidget            *m_basicOptsTab;
    QScrollArea        *m_otherScrollA;
    QWidget            *m_otherOptsTab;
    QWidget            *m_colorOpts;
    QCheckBox          *m_invertColors;
    
    SANE_Handle        m_saneHandle;
    
    QList<SaneOption*> m_optList;
    SaneOption        *m_optSource;
    SaneOption        *m_optNegative;
    SaneOption        *m_optFilmType;
    SaneOption        *m_optMode;
    SaneOption        *m_optDepth;
    SaneOption        *m_optRes;
    SaneOption        *m_optResY;
    SaneOption        *m_optTlX;
    SaneOption        *m_optTlY;
    SaneOption        *m_optBrX;
    SaneOption        *m_optBrY;
    SaneOption        *m_optPreview;
    SaneOption        *m_optGamR;
    SaneOption        *m_optGamG;
    SaneOption        *m_optGamB;
    QCheckBox         *m_splitGamChB;
    LabeledGamma      *m_commonGamma;
    SaneOption        *m_optWaitForBtn;
    
    QTimer              m_readValsTmr;
    
private slots:
    void checkInvert();
    void scheduleValReload();
    void valReload();
    void optReload();
    
    void slotTlXChanged(float);
    void slotTlYChanged(float);
    void slotBrXChanged(float);
    void slotBrYChanged(float);

private:
    void addRowToGridLayout(QGridLayout* l, QWidget* w, QWidget* labelWidget = 0);
    void addRowToGridLayout(QGridLayout* l, SaneOption* option);
};

}

#endif // SANE_OPTIONS_WIDGET_PRIVATE_H
