/* ============================================================
 *
 * Copyright (C) 2010 by Alexander Volkov <volkov0aa at gmail dot com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) version 3, or any
 * later version accepted by the membership of KDE e.V. (or its
 * successor approved by the membership of KDE e.V.), which shall
 * act as a proxy defined in Section 6 of version 3 of the license.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ============================================================ */

#include "sane_password_dialog.h"
#include "sane_password_dialog.moc"

#include <QLabel>
#include <QLineEdit>
#include <QDialogButtonBox>
#include <QVBoxLayout>
#include <QFormLayout>
#include <QFrame>
#include <QStyle>

namespace LibQSane
{

SanePasswordDialog::SanePasswordDialog(QWidget* parent)
{
    setWindowTitle(tr("Password"));
  
    QVBoxLayout* layout = new QVBoxLayout;
    setLayout(layout);
    
    m_promptLabel = new QLabel;
    m_promptLabel->setWordWrap(true);
    
    m_usernameEdit = new QLineEdit;
    m_passwordEdit = new QLineEdit;
    m_passwordEdit->setEchoMode(QLineEdit::Password);
    
    QFormLayout* formLayout = new QFormLayout;
    formLayout->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
    formLayout->setLabelAlignment(Qt::AlignLeft);
    formLayout->addRow(tr("Username:"), m_usernameEdit);
    formLayout->addRow(tr("Password:"), m_passwordEdit);
    
    QFrame* buttonsSeparator = new QFrame;
    buttonsSeparator->setFrameStyle(QFrame::HLine | QFrame::Sunken);
    
    QDialogButtonBox* buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
    connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
    
    layout->addWidget(m_promptLabel);
    layout->addSpacing(10);
    layout->addLayout(formLayout);
    layout->addWidget(buttonsSeparator);
    layout->addWidget(buttonBox);
}

void SanePasswordDialog::setPrompt(const QString& prompt)
{
    m_promptLabel->setText(prompt);
    setMinimumSize(sizeHint());
}
    
QString SanePasswordDialog::username() const
{
    return m_usernameEdit->text();
}

QString SanePasswordDialog::password() const
{
    return m_passwordEdit->text();
}

} // NameSpace LibQSane
